@@ -1,10 +1,10 @@
 package Spreadsheet::HTML;
 use strict;
 use warnings FATAL => 'all';
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 
 use Clone;
-use HTML::Element;
+use HTML::AutoTag;
 use Math::Matrix;
 use Spreadsheet::HTML::File::Loader;
 
@@ -74,7 +74,11 @@ sub process {
             # --headings
             $val = $args->{headings}->($val) if !$row and ref($args->{headings}) eq 'CODE';
 
-            $data->[$row][$col] = _element( $tag => $val, $args->{$tag} );
+            $data->[$row][$col] = { 
+                tag => $tag, 
+                (defined( $val ) ? (cdata => $val) : ()), 
+                (defined( $args->{$tag} ) ? (attr => $args->{$tag}) : ()),
+            };
         }
     }
 
@@ -108,21 +112,24 @@ sub _make_table {
     }
 
     my ($head, $foot, @body) = @{ $args{data} };
-    my $head_row  = [tr => $args{tr}, @$head];
-    my $foot_row  = [tr => $args{tr}, @{ $foot || [] }];
-    my @body_rows = map [tr => $args{tr}, @$_ ], @body;
-
-    my $table = HTML::Element->new_from_lol(
-        [table => $args{table},
-            ( $args{caption} ? [caption => {}, $args{caption}] : () ),
-            ( $args{tgroups} ? [ thead => $args{thead}, $head_row ]  : $head_row ),
-            ( $args{tgroups} ? [ tfoot => $args{tfoot}, $foot_row ]  : $foot ? $foot_row : () ),
-              $args{tgroups} ? [ tbody => $args{tbody}, @body_rows ] : @body_rows
-        ],
-    );
+    my $head_row  = { tag => 'tr', attr => $args{tr}, cdata => $head };
+    my $foot_row  = { tag => 'tr', attr => $args{tr}, cdata => $foot };
+    my @body_rows = map { tag => 'tr', attr => $args{tr}, cdata => $_ }, @body;
 
     my $encodes = exists $args{encodes} ? $args{encodes} : '';
-    return $table->as_HTML( $encodes, $args{indent} );
+    my $auto = HTML::AutoTag->new( encodes => $encodes, indent => $args{indent} );
+
+    # TODO: add caption back
+    # ( $args{caption} ? [caption => {}, $args{caption}] : () ),
+    return $auto->tag(
+        tag => 'table',
+        attr => $args{table},
+        cdata => [
+            ( $args{tgroups} ? { tag => 'thead', attr => $args{thead}, cdata => $head_row }  : $head_row ),
+            ( $args{tgroups} ? { tag => 'tfoot', attr => $args{tfoot}, cdata => $foot_row }  : $foot ? $foot_row : () ),
+              $args{tgroups} ? { tag => 'tbody', attr => $args{tbody}, cdata => [@body_rows] } : @body_rows
+        ],
+    );
 }
 
 sub _element {
@@ -194,9 +201,6 @@ named methods to control overall table orientation. These methods
 in turn accept a number of distinctly named attributes for directing
 what tags and attributes to use.
 
-Currently missing <col> and <colgroup> tags, row grouping and 
-fine grained cell attribute and content control.
-
 =head1 METHODS
 
 =over 4
@@ -346,9 +350,10 @@ There is currently no support for col and colgroup.
 
 =over 4
 
-=item * L<HTML::Tree>
+=item * L<HTML::AutoTag>
 
 Used to generate HTML. Handles indentation and HTML entity encoding.
+Uses L<Tie::Hash::Attribute> to handle rotation of class attributes.
 
 =item * L<Math::Matrix>
 
@@ -399,6 +404,40 @@ value contol, rotating attributes and totals/subtotals.
 
 =head1 BUGS AND LIMITATIONS
 
+Currently missing <col> and <colgroup> tags, row grouping and 
+fine grained cell attribute and content control.
+
+Benchmarks have improved since switching from HTML::Element
+to HTML::AutoTag but we are still a C- student at best.
+The following benchmark was performed by rendering a
+500x500 cell table 20 times:
+
+Before switch to HTML::AutoTag
+
+                     s/iter  S::H  H::E H::AT  H::T D::XT
+  Spreadsheet::HTML    8.58    --  -13%  -53%  -66%  -78%
+  HTML::Element        7.50   14%    --  -47%  -62%  -74%
+  HTML::AutoTag        4.01  114%   87%    --  -28%  -52%
+  HTML::Tiny           2.87  198%  161%   39%    --  -33%
+  DBIx::XHTML_Table    1.92  347%  291%  109%   50%    --
+
+
+After switch to HTML::AutoTag
+
+                    s/iter  H::E  S::H H::AT  H::T D::XT
+  HTML::Element       7.56    --  -34%  -46%  -60%  -74%
+  Spreadsheet::HTML   4.96   53%    --  -17%  -39%  -60%
+  HTML::AutoTag       4.12   84%   21%    --  -26%  -52%
+  HTML::Tiny          3.05  148%   63%   35%    --  -35%
+  DBIx::XHTML_Table   1.99  281%  150%  107%   53%    --
+
+Switching to HTML::Tiny would improve speed but this would
+complicate rotating attributes. The suggestion from these
+benchmarks is to do it the way DBIx::XHTML_Table does it:
+by complete brute force. This does not interest me ...
+if 1 second can be shaved off of HTML::AutoTag's time this
+would suffice.
+
 Please report any bugs or feature requests to either
 
 =over 4
@@ -431,11 +470,7 @@ This implementation is currently missing the following features:
 
 =item * emit col and colgroup tags
 
-=item * map client functions to cells by columns
-
-=item * assign "rotating" attrs to td tags by columns, etc.
-
-=item * what HTML::Element::as_HTML offers - including encoding and indention :(
+=item * map client attrs and functions to cells by columns/rows
 
 =back
 
